clc; clear;

% Time steps for trinomial tree.
N = 250;

% Table 1 of Bjerksund and Stensland  (2002)
K = 100;                 % Strike price.
S = [80 90 100 110 120];
r = [0.08 0.12 0.08 0.08];
sigma = [0.20 0.20 0.40 0.20];
T = [0.25 0.25 0.25 0.5];
b = -0.04;
q = r - b;

%% Calculate the trinomial tree, prices from the 1993 and 2003 papers
for j=1:4
    for k=1:5
        TreeCall(k,j) = TrinomialTree(S(k),K,r(j),q(j),sigma(j),T(j),N,'C','A');
        Call93(k,j)   = BjerkPrice1993(S(k),K,T(j),r(j),b,sigma(j),'C');
        Call02(k,j)   = BjerkPrice2002(S(k),K,T(j),r(j),b,sigma(j),'C');
        TreePut(k,j ) = TrinomialTree(S(k),K,r(j),q(j),sigma(j),T(j),N,'P','A');
        Put93(k,j)    = BjerkPrice1993(S(k),K,T(j),r(j),b,sigma(j),'P');
        Put02(k,j)    = BjerkPrice2002(S(k),K,T(j),r(j),b,sigma(j),'P');
    end
end

%% Output everything
clc;
fprintf('Table 1 of Bjerksund and Stensland (1993,2002) \n');
fprintf('Trinomial tree price, flat boundary price (1993) \n')
fprintf('and two-step boundary price (2002) \n')
fprintf('---------------------------------+-------------------------------\n')
fprintf('Spot          American Call      |           American Put        \n')
fprintf('Price   Tree    Price93  Price02 |     Tree    Price93  Price02  \n'); 
fprintf('---------------------------------+-------------------------------\n')
for j=1:4
    for k=1:5
        fprintf('%4.0f %8.2f %8.2f %8.2f  |  %8.2f %8.2f %8.2f \n',...
            S(k),TreeCall(k,j),Call93(k,j),Call02(k,j),...
            TreePut(k,j), Put93(k,j), Put02(k,j));
    end
    fprintf('---------------------------------+-------------------------------\n')
end

